program sdrSI2d_1d_t_omp
  ! computes AB*
  
  use sep                      ! Main SEP module
  use sep_timers_mod
  
  implicit none
  
  ! Include for FFTW
  include "fftw3.f"

  ! Memory variables
  real,allocatable   , dimension(:,:)  :: dataA_t,data_si_t
  complex,allocatable, dimension(:,:)  :: dataA_w,data_si_w
  
  real,allocatable   , dimension(:)    :: fft_r, dataB_t
  complex,allocatable, dimension(:)    :: fft_c, dataB_w

  ! Axis units
  integer                         :: nt,nw,ntau
  
  integer                         :: na1,na2,nb1,nb2
  real                            :: da1,da2,oa1,oa2,db1,db2,ob1,ob2

  integer                         :: ns,js,fs,nr,jr,fr
  real                            :: eps,scaler
  
  ! Type
  integer                         :: type                    ! 1: cross-spectrum, 2: coherence, 3: deconvolution (A/B), 4: deconvolution (A\B)
  
  ! Misc
  integer                         :: ii,jj,kk
  real, parameter                 :: pi=3.14159265
  logical                         :: verbose,internal,external

  ! FFTW plans  
  integer*8                       :: fft_for,fft_inv

  ! For timers
  logical                         :: logic
  integer                         :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  ! Open I/O headerfiles
  call sep_init()
  call from_param('verbose',verbose,.false.)
  call from_param('verb'   ,verbose,verbose)
  if (verbose) write(0,*) '-------------------------------------------'
  if (verbose) write(0,*) '         CORRELATE FILE A AND B            '
  if (verbose) write(0,*) '-------------------------------------------'
  if (verbose) write(0,*) '       i/o looping over traces in B        '

  if (exist_file("A")) then;
    call from_aux("A","n1", na1 ,1 )
    call from_aux("A","n2", na2 ,1 )
    call from_aux("A","d1", da1 ,1.)
    call from_aux("A","d2", da2 ,1.)
    call from_aux("A","o1", oa1 ,0.)
    call from_aux("A","o2", oa2 ,0.) 
  else
    call seperr("Need input file A")   
  end if
  ! Can I do r2c FFTW?
  if (floor(real(na1)/2.).ne.ceiling(real(na1)/2.)) call seperr("This would be a lot easier if n1 of file A is divisible by 2")
  

  if (exist_file("B")) then;
    call from_aux("B","n1", nb1 ,1 )
    call from_aux("B","n2", nb2 ,1 )
    call from_aux("B","d1", db1 ,1.)
    call from_aux("B","d2", db2 ,1.)
    call from_aux("B","o1", ob1 ,0.)
    call from_aux("B","o2", ob2 ,0.)    
  else
    call seperr("Need input file B")   
  end if
  ! Can I do r2c FFTW?
  if (floor(real(nb1)/2.).ne.ceiling(real(nb1)/2.)) call seperr("This would be a lot easier if n1 of file B is divisible by 2")
    
  ! Check if file A and B have similar time axis
  if (da1.ne.db1) call seperr("d1 differs in file A and B")
  if (oa1.ne.ob1) call seperr("o1 differs in file A and B")
  if (na1.ne.nb1) call seperr("n1 differs in file A and B")
  
  ! Sampling of axis on file A ('receivers')
  call from_param('nr',nr,na2)
  call from_param('jr',jr,  1)
  call from_param('fr',fr,  0)
  
  ! Sampling of axis on file B ('sources')
  call from_param('ns',ns,nb2)
  call from_param('js',js,  1)
  call from_param('fs',fs,  0)

  ! Error checks, check if files have equal sizes
  if (jr.lt.0) call seperr("jr.lt.0")
  if (fr.lt.0) call seperr("fr.lt.0")
  if (1+fr+(nr-1)*jr.gt.na2) call seperr('1+fr+(nr-1)*jr.gt.na2')
  
  if (js.lt.0) call seperr("js.lt.0")
  if (fs.lt.0) call seperr("fs.lt.0")
  if (1+fs+(ns-1)*js.gt.nb2) call seperr('1+fs+(ns-1)*js.gt.nb2')

  ! Correlation details
  call from_param("ntau"    , ntau   ,  na1  )
  call from_param("type"    ,type    ,  1    )  ! 1: cross-spectrum, 2: coherence, 3: deconvolution (A/B), 4: deconvolution (A\B)
  call from_param("eps"     , eps    ,  0.   )
  call from_param("external",external,.true. )
  call from_param("internal",internal,.false.)
  if (external.eq..false.)   internal = .true.
  if (verbose) then
    if (     internal) write(0,*) "internal correlation"
    if (.not.internal) write(0,*) "external correlation"
  end if

  ! internal or external circular correlation (internal)
  if (internal) then; nt=na1; else; nt=2*na1; end if
  
  ! Update History File
  call to_history  ("n1",2*ntau     )
  call to_history  ("d1",       da1 )
  call to_history  ("o1", -ntau*da1 )

  call to_history  ("o2", oa2+fr*da2 )
  call to_history  ("n2",     nr     )
  call to_history  ("d2",     jr*da2 )

  call to_history  ("o3", ob2+fs*db2 )
  call to_history  ("n3",     ns     )
  call to_history  ("d3",     js*db2 )
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Initialize FFTW routines
  nw = nt/2+1
  if (verbose) then
    write(0,*) "nt:",nt,'nw:',nw
  end if
  
  ! Allocate the arrays datain and dataout
  allocate( dataA_t  (nt,nr), dataA_w  (nw,nr) )
  allocate( data_si_t(nt,nr), data_si_w(nw,nr) )
  allocate( dataB_t  (nt   ), dataB_w  (nw   ) )
  allocate( fft_r    (nt   ), fft_c    (nw   ) )
  
  ! Prepare fftw plans
  call sfftw_plan_dft_r2c_1d(fft_for, nt,  fft_r , fft_c  , FFTW_ESTIMATE) ! Forward
  call sfftw_plan_dft_c2r_1d(fft_inv, nt,  fft_c , fft_r  , FFTW_ESTIMATE) ! Inverse
  scaler = 1./nt
  
  ! Read Data
  dataA_t  = 0.
  dataA_w  = 0.
  ! Read from file A
  if (internal) then ! (for internal correlation)
    do jj=1,nr
      call sreed_window("A",2,(/na1,na2/),(/na1/2,1/),(/ na2/2,fr+(jj-1)*jr/),(/1,1/),4,dataA_t(      1:na1/2,jj))
      call sreed_window("A",2,(/na1,na2/),(/na1/2,1/),(/     0,fr+(jj-1)*jr/),(/1,1/),4,dataA_t(na1/2+1:na1  ,jj))
    end do
  else
    do jj=1,nr ! (for external correlation)
      call sreed_window("A",2,(/na1,na2/),(/na1,1/),(/0,fr+(jj-1)*jr/),(/1,1/),4,dataA_t(1:na1,jj))
    end do
  end if

  ! Do all FFT's for the source side
  do jj=1,nr
    fft_r = dataA_t(:,jj)
    call sfftw_execute( fft_for )
    dataA_w(:,jj) = fft_c
  end do

  ! Now do for each source
  if (verbose) write(0,*) '-------------------------------------------'
  do kk=1,ns
    if (verbose) write(0,*) "Correlating:",kk,'/',ns

    dataB_t = 0.
    dataB_w = 0.
    data_si_t = 0.
    data_si_w = 0.
    
    ! Read source trace
    if (internal) then ! (for internal correlation)
      call sreed_window("B",2,(/nb1,nb2/),(/nb1/2,1/),(/ nb2/2,fs+(kk-1)*js/),(/1,1/),4,dataB_t(      1:nb1/2))
      call sreed_window("B",2,(/nb1,nb2/),(/nb1/2,1/),(/     0,fs+(kk-1)*js/),(/1,1/),4,dataB_t(nb1/2+1:nb1  ))
    else ! (for external correlation)
      call sreed_window("B",2,(/nb1,nb2/),(/nb1,1/),(/0,fs+(kk-1)*js/),(/1,1/),4,dataB_t(1:nb1))
    end if

    ! Do FFT's for the source side
    fft_r = dataB_t
    call sfftw_execute( fft_for )
    dataB_w = fft_c

    ! Already take complex conjugate source side
    do ii=1,nw
      dataB_w(ii) = conjg(dataB_w(ii))
    end do
    
    ! Interferometry
    select case(type)
      case(1)
        do jj=1,nr
          do ii=1,nw
            data_si_w(ii,jj) = dataA_w(ii,jj) * dataB_w(ii)
          end do
        end do
    end select
    
    ! Do IFFT's for the correlation result
    do jj=1,nr
      fft_c = data_si_w(:,jj)
      call sfftw_execute( fft_inv )
      data_si_t(:,jj) = fft_r
    end do

    do jj=1,nr
      call srite("out", data_si_t(1-ntau+nt:nt   ,jj), 4*ntau )
      call srite("out", data_si_t(1        :ntau ,jj), 4*ntau )
    end do

  end do
  
  
  ! Deallocate and close stuff
  if (verbose) write(0,*) '-------------------------------------------'
  call sfftw_destroy_plan(fft_for,fft_inv)
  deallocate( dataA_t, dataA_w, data_si_t , fft_r )
  deallocate( dataB_t, dataB_w, data_si_w , fft_c )
  call stop_timer_num (tm)
  call print_timers()
  call exit (0)
  
end program




